<?php
/* * * * * * * * * * * * * * * * * * * *
 *  ██████╗ █████╗  ██████╗ ███████╗
 * ██╔════╝██╔══██╗██╔═══██╗██╔════╝
 * ██║     ███████║██║   ██║███████╗
 * ██║     ██╔══██║██║   ██║╚════██║
 * ╚██████╗██║  ██║╚██████╔╝███████║
 *  ╚═════╝╚═╝  ╚═╝ ╚═════╝ ╚══════╝
 *
 * @author   : Daan van den Bergh
 * @url      : https://daan.dev/wordpress-plugins/caos/
 * @copyright: (c) 2020 Daan van den Bergh
 * @license  : GPL2v2 or later
 * * * * * * * * * * * * * * * * * * * */

defined('ABSPATH') || exit;

class CAOS_Frontend_Tracking
{
    /**
     * CAOS_Frontend_Tracking constructor.
     */
    public function __construct()
    {
        add_action('init', [$this, 'insert_tracking_code']);
        add_action('caos_process_settings', [$this, 'disable_display_features']);
        add_action('caos_process_settings', [$this, 'anonymize_ip']);
        add_action('caos_process_settings', [$this, 'linkid']);
        add_action('caos_process_settings', [$this, 'google_optimize']);
    }

    /**
     * Render the tracking code in it's selected locations
     */
    public function insert_tracking_code()
    {
        if (CAOS_OPT_COMPATIBILITY_MODE == 'woocommerce') {
            add_filter('woocommerce_google_analytics_script_src', array($this, 'return_analytics_js_url'), PHP_INT_MAX);
        } elseif (CAOS_OPT_COMPATIBILITY_MODE == 'monster_insights') {
            add_filter('monsterinsights_frontend_output_analytics_src', array($this, 'return_analytics_js_url'), PHP_INT_MAX);
        } elseif (CAOS_OPT_COMPATIBILITY_MODE == 'analytify') {
            add_filter('analytify_output_ga_js_src', array($this, 'return_analytics_js_url'), PHP_INT_MAX);
        } elseif (CAOS_OPT_COMPATIBILITY_MODE == 'exact_metrics') {
            add_filter('gadwp_analytics_script_path', array($this, 'return_analytics_js_url'), PHP_INT_MAX);
        } elseif (current_user_can('manage_options') && !CAOS_OPT_TRACK_ADMIN) {
            switch (CAOS_OPT_SCRIPT_POSITION) {
                case "footer":
                    add_action('wp_footer', array($this, 'show_admin_message'), CAOS_OPT_ENQUEUE_ORDER);
                    break;
                case "manual":
                    break;
                default:
                    add_action('wp_head', array($this, 'show_admin_message'), CAOS_OPT_ENQUEUE_ORDER);
                    break;
            }
        } else {
            /**
             * Allows WP DEV's to modify the output of the tracking code.
             *
             * E.g. add_action('caos_process_settings', 'your_function_name');
             */
            do_action('caos_process_settings');

            switch (CAOS_OPT_SCRIPT_POSITION) {
                case "footer":
                    add_action('wp_footer', array($this, 'render_tracking_code'), CAOS_OPT_ENQUEUE_ORDER);
                    break;
                case "manual":
                    break;
                default:
                    add_action('wp_head', array($this, 'render_tracking_code'), CAOS_OPT_ENQUEUE_ORDER);
                    break;
            }
        }

        if (CAOS_OPT_EXT_TRACK_AD_BLOCKERS == 'on') {
            add_action('wp_head', [$this, 'trigger_ad_blocker']);
        }
    }

    /**
     * Process disable display features setting.
     */
    public function disable_display_features()
    {
        if (CAOS_OPT_DISABLE_DISPLAY_FEAT !== 'on') {
            return;
        }

        if ($this->is_gtag()) {
            add_filter('caos_gtag_config', function($config, $trackingId) {
                return $config + array('displayFeaturesTask' => null);
            }, 10, 2);
        }

        add_filter('caos_analytics_before_send', function($config) {
            $option = array(
                'display_features' => "ga('set', 'displayFeaturesTask', null);"
            );

            return $config + $option;
        });
    }

    /**
     * Process Anonymize IP setting.
     */
    public function anonymize_ip()
    {
        if (CAOS_OPT_ANONYMIZE_IP !== 'on') {
            return;
        }

        if ($this->is_gtag()) {
            add_filter('caos_gtag_config', function($config, $trackingId) {
                return $config + array('anonymize_ip' => true);
            }, 10, 2);
        }

        add_filter('caos_analytics_before_send', function($config) {
            $option = array(
                'anonymize' => "ga('set', 'anonymizeIp', true);"
            );

            return $config + $option;
        });
    }

    /**
     * Enhanced Link Attribution
     *
     * TODO: Set samesite flag as soon as it's available. (https://developers.google.com/analytics/devguides/collection/analyticsjs/enhanced-link-attribution)
     */
    public function linkid()
    {
        if (CAOS_OPT_EXT_LINKID !== 'on') {
            return;
        }

        if ($this->is_gtag()) {
            add_filter('caos_gtag_config', function ($config, $tracking_id) {
                return $config + ['linkid', [
                        'cookie_name'  => 'caos_linkid',
                        'cookie_flags' => 'samesite=none;secure'
                    ]
                ];
            }, 10, 2);
        }

        add_filter('caos_analytics_before_send', function($config) {
            $option = [
                'linkid' => "ga('require', 'linkid', { 'cookieName':'caosLinkid', 'cookieFlags':'samesite=none;secure' });"
            ];

            return $config + $option;
        });
    }

    /**
     * Google Optimize
     */
    public function google_optimize()
    {
        if (CAOS_OPT_EXT_OPTIMIZE !== 'on') {
            return;
        }

        $optimize_id = CAOS_OPT_EXT_OPTIMIZE_ID;

        if (!$optimize_id) {
            return;
        }

        if ($this->is_gtag()) {
            add_filter('caos_gtag_config', function ($config, $tracking_id) use ($optimize_id) {
                return $config + [ 'optimize_id' => $optimize_id ];
            }, 10, 2);
        }

        add_filter('caos_analytics_before_send', function ($config) use ($optimize_id) {
            $option = [
                'optimize' => "ga('require', '$optimize_id');"
            ];

            return $config + $option;
        });
    }

    /**
     * @return bool
     */
    private function is_gtag()
    {
        return CAOS_OPT_REMOTE_JS_FILE == 'gtag.js';
    }

    /**
     * Render a HTML comment for logged in Administrators in the source code.
     */
    public function show_admin_message()
    {
        echo "<!-- " . __('This site is using CAOS. You\'re logged in as an administrator, so we\'re not loading the tracking code.', 'host-analyticsjs-local') . " -->\n";
    }

    /**
     * Render the URL of the cached local-ga.js file
     *
     * @return string
     */
    public function return_analytics_js_url()
    {
        return CAOS_LOCAL_FILE_URL;
    }

    /**
     * Generate tracking code and add to header/footer (default is header)
     */
    public function render_tracking_code()
    {
        if (!CAOS_OPT_TRACKING_ID) {
            return;
        }

        echo "<!-- " . __('This site is running CAOS for Wordpress', 'host-analyticsjs-local') . " -->\n";

        if (CAOS_OPT_REMOTE_JS_FILE == 'gtag.js' || (CAOS_OPT_SNIPPET_TYPE == 'async' && CAOS_OPT_REMOTE_JS_FILE != 'ga.js')) {
            $urlId            = CAOS_OPT_REMOTE_JS_FILE == 'gtag.js' ? "?id=" . CAOS_OPT_TRACKING_ID : '';
            $snippetType      = CAOS_OPT_SNIPPET_TYPE;
            $localFileUrl     = CAOS_LOCAL_FILE_URL . $urlId;
            $scriptAttributes = CAOS_OPT_REMOTE_JS_FILE == 'gtag.js'
                ? apply_filters('caos_gtag_script_element_attributes', '')
                : apply_filters('caos_analytics_script_element_attributes', '');

            echo "<script $snippetType src='$localFileUrl' $scriptAttributes></script>";
        }

        if (CAOS_OPT_ALLOW_TRACKING == 'cookie_has_value' && CAOS_OPT_COOKIE_NAME && CAOS_OPT_COOKIE_VALUE) {
            $this->get_tracking_code_template('cookie-value');
        }

        if (CAOS_OPT_REMOTE_JS_FILE == 'gtag.js') {
            $this->get_tracking_code_template('gtag');
        } else {
            $this->get_tracking_code_template('analytics');
        }
    }

    /**
     * @param $name
     */
    public function get_tracking_code_template($name)
    {
        return include CAOS_PLUGIN_DIR . 'templates/frontend-tracking-code-' . $name . '.phtml';
    }

    /**
     *
     */
    public function trigger_ad_blocker()
    {
        wp_enqueue_script('caos-track-ad-blockers', plugins_url('assets/js/detect-ad-block.js', CAOS_PLUGIN_FILE), [ 'jquery' ], CAOS_STATIC_VERSION, true);
        wp_add_inline_script('caos-track-ad-blockers', $this->send_ad_blocker_result());
    }

    /**
     *
     */
    private function send_ad_blocker_result()
    {
        $url = site_url('wp-json/caos/v1/block/detect');
        $script = "jQuery(document).ready(function ($) { var caos_detect_ad_blocker = 1; if (document.getElementById('caos-detect-ad-block')) { caos_detect_ad_blocker = 0; } $.ajax({ method: 'GET', url: '$url', data: { result: caos_detect_ad_blocker } }) });";

        return $script;
    }
}
